
{**********************************************************************
 Package pl_SynapseVS
 From PilotLogic Software House(https://www.pilotlogic.com/)
 This file is part of CodeTyphon Studio
***********************************************************************}

Unit vsComPort;

{$mode objfpc}{$H+}

Interface

Uses
   {$IFDEF UNIX}
  Classes,
{$IFDEF UseCThreads}
  cthreads,
{$ENDIF}
   {$ELSE}
   Windows, Classes,
   {$ENDIF}
   SysUtils, LResources, Forms, Controls, Graphics, Dialogs,
   synaser, vsComPortbase, vsComPortsetup;

Type
   TvsComPort = Class;

   TComPortReadThread = Class(TThread)
   Public
      MustDie: Boolean;
      Owner: TvsComPort;
   Protected
      Procedure CallEvent;
      Procedure Execute; Override;
   Published
      Property Terminated;
   End;

   TvsComPort = Class(TComponent)
   Private
      FActive: Boolean;
      FSynSer: TBlockSerial;
      FDevice: String;
      FBaudRate: TBaudRate;
      FDataBits: TDataBits;
      FParity: TParity;
      FStopBits: TStopBits;
      FSoftflow, FHardflow: Boolean;
      FFlowControl: TFlowControl;
      FRcvLineCRLF: Boolean;
      FOnRxData: TNotifyEvent;
      FOnStatus: TStatusEvent;
      ReadThread: TComPortReadThread;
      Procedure DeviceOpen;
      Procedure DeviceClose;
      Procedure ComException(str: String);
   Protected
      Procedure SetActive(state: Boolean);
      Procedure SetBaudRate(br: TBaudRate);
      Procedure SetDataBits(db: TDataBits);
      Procedure SetParity(pr: TParity);
      Procedure SetFlowControl(fc: TFlowControl);
      Procedure SetStopBits(sb: TStopBits);
   Public
      Constructor Create(AOwner: TComponent); Override;
      Destructor Destroy; Override;
      Procedure Open;
      Procedure Close;
      // show a port settings dialog form
      Procedure ShowSetupDialog;
      // read data from port
      Function DataAvailable: Boolean;
      Function ReadData: String;
      // write data to port
      Function WriteData(Data: String): Integer;
      Function WriteBuffer(Var buf; size: Integer): Integer;
      // read pin states
      Function ModemSignals: TModemSignals;
      Function GetDSR: Boolean;
      Function GetCTS: Boolean;
      Function GetRing: Boolean;
      Function GetCarrier: Boolean;
      // set pin states
      Procedure SetDTR(OnOff: Boolean);
      Procedure SetRTS(OnOff: Boolean);
   Published
      Property Active: Boolean Read FActive Write SetActive;
      Property BaudRate: TBaudRate Read FBaudRate Write SetBaudRate;
      Property DataBits: TDataBits Read FDataBits Write SetDataBits;
      Property Parity: TParity Read FParity Write SetParity;
      Property FlowControl: TFlowControl Read FFlowControl Write SetFlowControl;
      Property StopBits: TStopBits Read FStopBits Write SetStopBits;
      Property SynSer: TBlockSerial Read FSynSer Write FSynSer;
      Property Device: String Read FDevice Write FDevice;
      Property RcvLineCRLF: Boolean Read FRcvLineCRLF Write FRcvLineCRLF;
      Property OnRxData: TNotifyEvent Read FOnRxData Write FOnRxData;
      Property OnStatus: TStatusEvent Read FOnStatus Write FOnStatus;
   End;

Procedure EditComPort(ComPort: TvsComPort);

Implementation

Procedure EditComPort(ComPort: TvsComPort);
Begin
   With TComSetupFrm.Create(nil) Do
   Begin
      ComComboBox1.Text := ComPort.Device;
      ComComboBox2.Text := BaudRateToStr(ComPort.BaudRate);
      ComComboBox3.Text := DataBitsToStr(ComPort.DataBits);
      ComComBoBox4.Text := StopBitsToStr(ComPort.StopBits);
      ComComBoBox5.Text := ParityToStr(ComPort.Parity);
      ComComBoBox6.Text := FlowControlToStr(ComPort.FlowControl);

      If ShowModal = mrOk Then
      Begin
         ComPort.Close;
         ComPort.Device := ComComboBox1.Text;
         ComPort.BaudRate := StrToBaudRate(ComComboBox2.Text);
         ComPort.DataBits := StrToDataBits(ComComboBox3.Text);
         ComPort.StopBits := StrToStopBits(ComComboBox4.Text);
         ComPort.Parity := StrToParity(ComComboBox5.Text);
         ComPort.FlowCOntrol := StrToFlowControl(ComComboBox6.Text);
         ComPort.Open;
      End;
      Free;
   End;
End;


//===================== TvsComPort ========================================

Constructor TvsComPort.Create(AOwner: TComponent);
Begin
   Inherited;
   ReadThread := nil;

   If csDesigning In ComponentState Then
   Begin
      FSynSer := nil;
   End Else Begin
      FSynSer := TBlockSerial.Create;
      FSynSer.LinuxLock := False;
   End;

   FHardflow := False;
   FSoftflow := False;
   FFlowControl := fcNone;
   {$IFDEF UNIX}
   FDevice:='/dev/ttyS0';
   {$ELSE}
   FDevice := 'COM1';
   {$ENDIF}
   FRcvLineCRLF := False;
   FBaudRate:=br115200;
End;

Destructor TvsComPort.Destroy;
Begin
   Close;
   If FSynSer = nil Then
      FSynSer.Free;
   Inherited;
End;

Procedure TvsComPort.Close;
Begin
   Active := False;
End;

Procedure TvsComPort.DeviceClose;
Begin
   If FSynSer = nil Then
      Exit;

   // flush device
   If FSynSer.Handle <> INVALID_HANDLE_VALUE Then
   Begin
      FSynSer.Flush;
      FSynSer.Purge;
   End;

   // stop capture thread
   If ReadThread <> nil Then
   Begin
      ReadThread.FreeOnTerminate := False;
      ReadThread.MustDie := True;
      While Not ReadThread.Terminated Do
      Begin
         Application.ProcessMessages;
      End;
      ReadThread.Free;
      ReadThread := nil;
   End;

   // close device
   If FSynSer.Handle <> INVALID_HANDLE_VALUE Then
   Begin
      FSynSer.Flush;
      FSynSer.CloseSocket;
   End;
End;

Function TvsComPort.DataAvailable: Boolean;
Begin        
   Result := False;
   If FSynSer = nil Then
      Exit;

   If FSynSer.Handle = INVALID_HANDLE_VALUE Then
   Begin
      Result := False;
      exit;
   End;
   Result := FSynSer.CanReadEx(0);
End;

Procedure TvsComPort.Open;
Begin
   If FSynSer = nil Then
      Exit;

   // Initialize OnStatus;
   If Assigned(OnStatus) Then
      SynSer.OnStatus := OnStatus;
   Active := True;
End;

Procedure TvsComPort.ShowSetupDialog;
Begin
   EditComPort(self);
End;

Procedure TvsComPort.DeviceOpen;
Begin
   If FSynSer = nil Then
      Exit;

   FSynSer.Connect(FDevice);
   If FSynSer.Handle = INVALID_HANDLE_VALUE Then
      Raise Exception.Create('Could not open device ' + FSynSer.Device);

   FSynSer.Config(ConstsBaud[FBaudRate],
      ConstsBits[FDataBits],
      ConstsParity[FParity],
      ConstsStopBits[FStopBits],
      FSoftflow, FHardflow);

   // Launch Thread
   ReadThread := TComPortReadThread.Create(True);
   ReadThread.Owner := Self;
   ReadThread.MustDie := False;
   //  ReadThread.Resume;   --> deprecated
   ReadThread.Start;
End;


Function TvsComPort.ReadData: String;
Begin
   Result := '';
   If FSynSer = nil Then
      Exit;

   If FSynSer.Handle = INVALID_HANDLE_VALUE Then
      ComException('can not read from a closed port.');
   If FRcvLineCRLF Then
      Result := FSynSer.RecvString(0)
   Else
      Result := FSynSer.RecvPacket(0);
End;

Procedure TvsComPort.SetActive(state: Boolean);
Begin
   If state = FActive Then
      exit;

   If state Then
      DeviceOpen
   Else
      DeviceClose;

   FActive := state;
End;

Procedure TvsComPort.SetBaudRate(br: TBaudRate);
Begin
   If FSynSer <> nil Then
      If FSynSer.Handle <> INVALID_HANDLE_VALUE Then
      Begin
         FSynSer.Config(ConstsBaud[FBaudRate], ConstsBits[FDataBits], ConstsParity[FParity],
            ConstsStopBits[FStopBits], FSoftflow, FHardflow);
      End;
   FBaudRate := br;
End;

Procedure TvsComPort.SetDataBits(db: TDataBits);
Begin
   If FSynSer <> nil Then
      If FSynSer.Handle <> INVALID_HANDLE_VALUE Then
      Begin
         FSynSer.Config(ConstsBaud[FBaudRate], ConstsBits[FDataBits], ConstsParity[FParity],
            ConstsStopBits[FStopBits], FSoftflow, FHardflow);
      End;
   FDataBits := db;
End;

Procedure TvsComPort.SetFlowControl(fc: TFlowControl);
Begin
   If fc = fcNone Then
   Begin
      FSoftflow := False;
      FHardflow := False;
   End Else
   If fc = fcXonXoff Then
   Begin
      FSoftflow := True;
      FHardflow := False;
   End Else
   If fc = fcHardware Then
   Begin
      FSoftflow := False;
      FHardflow := True;
   End;

   If FSynSer <> nil Then
      If FSynSer.Handle <> INVALID_HANDLE_VALUE Then
      Begin
         FSynSer.Config(ConstsBaud[FBaudRate], ConstsBits[FDataBits], ConstsParity[FParity],
            ConstsStopBits[FStopBits], FSoftflow, FHardflow);
      End;
   FFlowControl := fc;
End;

Procedure TvsComPort.SetParity(pr: TParity);
Begin
   If FSynSer <> nil Then
      If FSynSer.Handle <> INVALID_HANDLE_VALUE Then
      Begin
         FSynSer.Config(ConstsBaud[FBaudRate], ConstsBits[FDataBits], ConstsParity[FParity],
            ConstsStopBits[FStopBits], FSoftflow, FHardflow);
      End;
   FParity := pr;
End;

Procedure TvsComPort.SetStopBits(sb: TStopBits);
Begin
   If FSynSer <> nil Then
      If FSynSer.Handle <> INVALID_HANDLE_VALUE Then
      Begin
         FSynSer.Config(ConstsBaud[FBaudRate], ConstsBits[FDataBits], ConstsParity[FParity],
            ConstsStopBits[FStopBits], FSoftflow, FHardflow);
      End;
   FStopBits := sb;
End;

Function TvsComPort.WriteBuffer(Var buf; size: Integer): Integer;
Begin
   Result := 0;
   If FSynSer <> nil Then
      Result := FSynSer.SendBuffer(Pointer(@buf), size);
End;

Function TvsComPort.WriteData(Data: String): Integer;
Begin
   Result := length(Data);
   If FSynSer <> nil Then
      FSynSer.SendString(Data);
End;

Function TvsComPort.ModemSignals: TModemSignals;
Begin
   Result := []; 
   If FSynSer = nil Then
      Exit;

   If FSynSer.CTS Then
      Result := Result + [msCTS];
   If FSynSer.carrier Then
      Result := Result + [msCD];
   If FSynSer.ring Then
      Result := Result + [msRI];
   If FSynSer.DSR Then
      Result := Result + [msDSR];
End;

Function TvsComPort.GetDSR: Boolean;
Begin
   Result := False;
   If FSynSer <> nil Then
      Result := FSynSer.DSR;
End;

Function TvsComPort.GetCTS: Boolean;
Begin
   Result := False;
   If FSynSer <> nil Then
      Result := FSynSer.CTS;
End;

Function TvsComPort.GetRing: Boolean;
Begin
   Result := False;
   If FSynSer <> nil Then
      Result := FSynSer.ring;
End;

Function TvsComPort.GetCarrier: Boolean;
Begin
   Result := False;
   If FSynSer <> nil Then
      Result := FSynSer.carrier;
End;

Procedure TvsComPort.SetDTR(OnOff: Boolean);
Begin
   If FSynSer <> nil Then
      FSynSer.DTR := OnOff;
End;

Procedure TvsComPort.SetRTS(OnOff: Boolean);
Begin
   If FSynSer <> nil Then
      FSynSer.RTS := OnOff;
End;

Procedure TvsComPort.ComException(str: String);
Begin
   Raise Exception.Create('ComPort error: ' + str);
End;

//==================== TComPortReadThread ==========================================

Procedure TComPortReadThread.CallEvent;
Begin
   If Assigned(Owner.FOnRxData) Then
   Begin
      Owner.FOnRxData(Owner);
   End;
End;

Procedure TComPortReadThread.Execute;
Begin
   Try
      While Not MustDie Do
      Begin
         If Owner.FSynSer.CanReadEx(100) Then
            Synchronize(@CallEvent);
      End;
   Finally
      Terminate;
   End;
End;

End.
